function H = butter_lo_design(f, Fc, A, Stage1, Stage2, Stage3, Stage4, Stage5)
%
% H = b_lo_des(f, Fc, A, Stage1[, Stage2[, Stage3[, Stage4[, Stage5]]]])
%
%   Helps design an electronic circuit to build even-order, low-pass
% Butterworth filters of orders 2, 4, 6, 8, 10.  Plots the magnitude of
% the resulting frequency response.
%   See [Thomas Kugelstadt, "Chapter 16: Active
% Filter Design Techniques," Literature Number SLOA088, Texas Instruments
% Incorporated, Post Office Box 655303, Dallas, Texas  75265, 2001.
% Excerpted from "Op Amps for Everyone," Literature Number SLOD006A,
% Texas Instruments.  Available on the Internet at: http://www.ti.com].
%
% f:      Frequency axis (Hertz) for all calculations and plotting (vector).
% A:      Overall circuit gain (>=1).  Applied in 1st stage.
% Fc:     Desired cutoff frequency in Hertz (scalor).
% StageX: Up to 5 StageX arguments (one per stage) are permitted.  For each stage,
%         StageX is a vector of 1-4 elements, corresponding to C1, C2,
%         R1 and R2, respectively.  C1 must be supplied (in Farads) and
%         is the first vector element.  If a second argument is supplied,
%         then it is C2 (in Farads).  If C2 is not supplied, it is set
%         equal to C1.  If
%         a third argument is supplied, it is R1 (Ohms).  If not supplied,
%         it is set as required to form a Butterworth filter.  If a fourth
%         argument is supplied, it is R2 (Ohms).  If not supplied, it is
%         set as required to form a Butterworth filter.
%
% H: Resulting (complex-valued) frequency response corresponding to the 
%    frequency axis f.
%
% USAGE RECOMENDATIONS:  To build a filter, initially call script with only
%   C1 and C2 specified for each stage.  Use the recommended
%   R1 and R2 values to find R1's and R2's that are manufactured.  Call the
%   script a second time with all values to see the resultant nominal frequency
%   response.

% Table of ai values.  For even-order Butterworth, all bi values equal 1.
aiTable = [1.4142    NaN    NaN    NaN    NaN;  % For 1-stage filter.
           1.8478 0.7654    NaN    NaN    NaN;  % For 2-stage filter.
           1.9319 1.4142 0.5176    NaN    NaN;  % For 3-stage filter.
           1.9616 1.6629 1.1111 0.3902    NaN;  % For 4-stage filter.
           1.9754 1.7820 1.4142 0.9080 0.3129]; % For 5-stage filter.

% Determine the number of stages.
Nstage = nargin - 3;
if Nstage<1 | Nstage>5, error('Must have 1-5 stages.'); end
if A<1, error('"A" must be >= 1.'); end

% Extract/develop parameters for each stage and build the frequency response.

H = ones( 1, length(f) );  % Initialize frequency response to unity.
w = 2*pi*f;                % Convert to radian frequency.
MagBig = A;                % Cascade passband gain.

for S = 1:Nstage

  if S>1, A = 1; end  % Set gain to one after first stage.
  
  % Prepare to parse parameter vector.
  eval(['Param = Stage' int2str(S) ';']);  % Copy parameters to a scratch vector.
  if length(Param)<1, error(['Stage ' int2str(S) ' parameter list < 1.']); end;
  if length(Param)>4, error(['Stage ' int2str(S) ' parameter list > 4.']); end;
  ai = aiTable(Nstage,S);

  % Parse parameter vector.
  % C1.
  C1 = Param(1);
  % C2.
  if length(Param)>1,  C2 = Param(2);  else,  C2 = C1; end
  % R1.
  if length(Param)>2,  R1 = Param(3);
  else
    a = ((2*pi*Fc)^3) * C2 * ( C1 + (1-A)*C2 );
    b = -ai * C2 * ((2*pi*Fc)^2);
    c = 2*pi*Fc;
    R1 = ( -b - sqrt(b*b - 4*a*c) ) / (2*a);
  end
  % R2.
  if length(Param)>3,  R2 = Param(4);
  else,  R2 = 1 / ( ((2*pi*Fc)^2)*R1*C1*C2 ); end

  % Print component values.
  fprintf('Stage %d: C1=%e F, C2=%e F, R1=%e Ohms, R2=%e Ohms\n', S, C1, C2, R1, R2);

  % Update frequency response.
  a = C1*(R1+R2) + (1-A)*R1*C2;
  b = R1 * R2 * C1 * C2;
  Hstage = A ./ (  1 + ( a*j*w ) - ( b*w.*w )  );
  H = H .* Hstage;
end

% Now, plot the frequency response magnitude.
plot( f, abs(H) )
xlabel('Frequency in Hertz')
ylabel('Frequency Response Magnitude')

L1 = find( abs(H) <= (MagBig*sqrt(2)/2) );
hold on, plot([f(L1(1)) f(L1(1))], [0 MagBig], 'm'), hold off
Thing = sprintf('Cut-off at %0.2f Hz, Passband Gain = %0.2f', f(L1(1)), MagBig);
text( max(f)/5, max(abs(H))/3, Thing);
title('Butterworth Low-Pass Filter');

figure(gcf);

return
